/**
 * PlotPane.java
 */
package UTEStatsViewer;
import java.awt.*;
import java.awt.event.*;
import com_ibm_us_bolmar.GraphicPlotter.*;

/**
 * A PlotPane containing the following objects in a standard layout.
 * <ul>
 * <li> a GraphicPane
 * <li> x-axis and y-axis scales
 * <li> x-axis and y-axis labels
 * <li> scrollbars
 * <li> a LegendPane
 * <li> a LegendLabel
 * </ul>
 */
// extends Panel instead of Container because when extends Container
// non-visible scrollbars sometimes partially show
public class PlotPane extends com_ibm_us_bolmar.AWTX.Panel
      implements ScrollPositioner
{
   /** The Scrollbars of the GraphicPane and AxisScales. */
   MagnifyingScrollbar xScrollbar, yScrollbar;

   /** Magnifiable version of the GraphicPane. */
   private MagnifyingPane magnifiableGraphicPane;


   /** The GraphicPane. */
   GraphicPane graphicPane = new GraphicPane();

   /** Return the GraphicPane of this frame. */
   public final GraphicPane getGraphicPane() {return graphicPane;}


   /** The LegendPane. */
   LegendPane legend = new LegendPane();

   /** Return the body of the LegendPane of this Frame. */
   public final Container getLegendContainer() {return legend.getBody();}


   // Initialize a PlotPane (invoked by the constructors after they
   // do the initilization that depends on the consructor parameters).
   private final void init() {
      // Create the scrollbars and the magnifiable GraphicPane.
      if (xScrollbar == null) {
         xScrollbar = new MagnifyingScrollbar(Scrollbar.HORIZONTAL);
      }
      if (yScrollbar == null) {
         yScrollbar = new MagnifyingScrollbar(Scrollbar.VERTICAL);
      }
      magnifiableGraphicPane =
            new MagnifyingPane(graphicPane, xScrollbar, yScrollbar);

      // Have this container use GridBagLayout.
      super.setLayout(new GridBagLayout());

      // Add default components to self.
      addDefaultComponents();
   }


   /** Construct a PlotPane with default magnified coordinate value. */
   public PlotPane() {
      init();
   }


   /**
    * Construct a PlotPane with a given maximum magnified coordinate
    * value.
    */
   public PlotPane(int magnifiedCoordinateMax) {
      xScrollbar = new MagnifyingScrollbar(Scrollbar.HORIZONTAL,
            magnifiedCoordinateMax);
      yScrollbar = new MagnifyingScrollbar(Scrollbar.VERTICAL,
            magnifiedCoordinateMax);
      init();
   }


   /**
    * Add a graphic element to the GraphicPane.
    * @return the Graphic argument
    */
   public final Graphic addGraphic(Graphic comp) {
      graphicPane.addGraphic(comp);
      return comp;
   }


   /**
    * Add a graphic category to the LegendPane.
    * @return the Component argument
    */
   public final Component addLegend(Component comp) {
      legend.add(comp);
      return comp;
   }


   /**
    * Add the components that are always present.
    */
   private void addDefaultComponents() {
      GridBagConstraints layoutConstraints = new GridBagConstraints();

      // Add legend container.
      layoutConstraints.gridx = 0;
      layoutConstraints.gridy = 1;
      layoutConstraints.gridwidth = 4;
      layoutConstraints.gridheight = 1;
      layoutConstraints.fill = GridBagConstraints.HORIZONTAL;
      layoutConstraints.anchor = GridBagConstraints.SOUTH;
      add(legend, layoutConstraints);
      legend.addMouseListener(FocusAtMouse.getInstance());

      // Add the view.
      layoutConstraints.gridx = 2;
      layoutConstraints.gridy = 2;
      layoutConstraints.gridwidth = 1;
      layoutConstraints.gridheight = 1;
      layoutConstraints.fill = GridBagConstraints.BOTH;
      layoutConstraints.anchor = GridBagConstraints.CENTER;
      layoutConstraints.weighty = 1.0;
      layoutConstraints.weightx = 1.0;
      add(magnifiableGraphicPane, layoutConstraints);
      layoutConstraints.weighty = 0.0;
      layoutConstraints.weightx = 0.0;
      graphicPane.addMouseListener(new FocusAtMouse(magnifiableGraphicPane));
          // must addMouseListener to the graphicPane rather than the
          // magnifiableGraphicPane because graphicPane adds its own mouse listener

      // Add the vertical scrollbar.
      layoutConstraints.gridx = 3;
      layoutConstraints.gridy = 2;
      layoutConstraints.gridwidth = 1;
      layoutConstraints.gridheight = 1;
      layoutConstraints.fill = GridBagConstraints.VERTICAL;
      layoutConstraints.anchor = GridBagConstraints.WEST;
      add(yScrollbar, layoutConstraints);

      // Add the horizontal scrollbar.
      layoutConstraints.gridx = 2;
      layoutConstraints.gridy = 4;
      layoutConstraints.gridwidth = 1;
      layoutConstraints.gridheight = 1;
      layoutConstraints.fill = GridBagConstraints.HORIZONTAL;
      layoutConstraints.anchor = GridBagConstraints.NORTH;
      add(xScrollbar, layoutConstraints);
   }


   private MagnifyingPane xScalePane;
   /** Add an X-axis (horizontal) scale. */
   public void addXScale(AxisScale comp) {
      xScalePane = new MagnifyingPane(comp, xScrollbar, null);
      GridBagConstraints layoutConstraints = new GridBagConstraints(); 
      layoutConstraints.gridx = 2;
      layoutConstraints.gridy = 3;
      layoutConstraints.gridwidth = 1;
      layoutConstraints.gridheight = 1;
      layoutConstraints.fill = GridBagConstraints.HORIZONTAL;
      layoutConstraints.anchor = GridBagConstraints.NORTH;
      add(xScalePane, layoutConstraints);
      graphicPane.setXScale(comp);
      xScalePane.addMouseListener(FocusAtMouse.getInstance());
   }

   /** Remove the X-axis scale. */
   public void removeXScale() {
      if (xScalePane!= null) {
         xScalePane.removeMouseListener(FocusAtMouse.getInstance());
         super.remove(xScalePane);
         xScrollbar.remove(xScalePane);
         xScalePane = null;
         graphicPane.setXScale(null);
      }
   }

   /** Return the X-axis scale. */
   public AxisScale getXScale() {
      return graphicPane.getXScale();
   }


   private AxisLabel xAxisLabelDefault = new AxisLabelX();
   AxisLabel xAxisLabel = xAxisLabelDefault;
   /** Add an X-axis (horizontal) label. */
   public void addXLabel(AxisLabel comp) {
      xAxisLabel = comp;
      GridBagConstraints layoutConstraints = new GridBagConstraints(); 
      layoutConstraints.gridx = 2;
      layoutConstraints.gridy = 5;
      layoutConstraints.gridwidth = 1;
      layoutConstraints.gridheight = 1;
      layoutConstraints.fill = GridBagConstraints.HORIZONTAL;
      layoutConstraints.anchor = GridBagConstraints.SOUTH;
      add(xAxisLabel, layoutConstraints);
      xAxisLabel.addMouseListener(FocusAtMouse.getInstance());
   }

   /** Remove the X-axis label. */
   public void removeXLabel() {
      if (xAxisLabel != xAxisLabelDefault) {
         xAxisLabel.removeMouseListener(FocusAtMouse.getInstance());
         super.remove(xAxisLabel);
         xAxisLabel = xAxisLabelDefault;
      }
   }


   /** Return the Y-axis scale. */
   public AxisScale getYScale() {
      return graphicPane.getYScale();
   }

   private MagnifyingPane yScalePane;
   /** Add an Y-axis (vertical) scale. */
   public void addYScale(AxisScale comp) {
      yScalePane = new MagnifyingPane(comp, null, yScrollbar);
      GridBagConstraints layoutConstraints = new GridBagConstraints(); 
      layoutConstraints.gridx = 1;
      layoutConstraints.gridy = 2;
      layoutConstraints.gridwidth = 1;
      layoutConstraints.gridheight = 1;
      layoutConstraints.fill = GridBagConstraints.VERTICAL;
      layoutConstraints.anchor = GridBagConstraints.EAST;
      add(yScalePane, layoutConstraints);
      graphicPane.setYScale(comp);
      yScalePane.addMouseListener(FocusAtMouse.getInstance());
   }

   /** Remove the Y-axis scale. */
   public void removeYScale() {
      if (yScalePane != null) {
         yScalePane.removeMouseListener(FocusAtMouse.getInstance());
         super.remove(yScalePane);
         yScrollbar.remove(yScalePane);
         yScalePane = null;
         graphicPane.setYScale(null);
      }
   }


   private AxisLabel yAxisLabelDefault = new AxisLabelY();
   AxisLabel yAxisLabel = yAxisLabelDefault;
   /** Add an Y-axis (vertical) label. */
   public void addYLabel(AxisLabel comp) {
      yAxisLabel = comp;
      GridBagConstraints layoutConstraints = new GridBagConstraints(); 
      layoutConstraints.gridx = 0;
      layoutConstraints.gridy = 2;
      layoutConstraints.gridwidth = 1;
      layoutConstraints.gridheight = 1;
      layoutConstraints.fill = GridBagConstraints.VERTICAL;
      layoutConstraints.anchor = GridBagConstraints.WEST;
      add(yAxisLabel, layoutConstraints);
      yAxisLabel.addMouseListener(FocusAtMouse.getInstance());
   }

   /** Remove the Y-axis label. */
   public void removeYLabel() {
      if (yAxisLabel != yAxisLabelDefault) {
         yAxisLabel.removeMouseListener(FocusAtMouse.getInstance());
         super.remove(yAxisLabel);
         yAxisLabel = yAxisLabelDefault;
      }
   }


   Component legendLabel;
   /** Add a legend label. */
   public void addLegendLabel(Component comp) {
      legendLabel = comp;
      GridBagConstraints layoutConstraints = new GridBagConstraints(); 
      layoutConstraints.gridx = 0;
      layoutConstraints.gridy = 0;
      layoutConstraints.gridwidth = 4;
      layoutConstraints.gridheight = 1;
      layoutConstraints.fill = GridBagConstraints.HORIZONTAL;
      layoutConstraints.anchor = GridBagConstraints.SOUTH;
      add(comp, layoutConstraints);
      comp.addMouseListener(FocusAtMouse.getInstance());
   }

   /** Remove the legend label. */
   public void removeLegendLabel() {
      if (legendLabel != null) {
         legendLabel.removeMouseListener(FocusAtMouse.getInstance());
         super.remove(legendLabel);
         legendLabel = null;;
      }
   }


   /**
    * Remove all components for which this class has a public add
    * and remove all components from the GraphicPane and
    * LegendPane of this frame.
    */
   public synchronized void removeAll() {
      legend.removeAll();
      graphicPane.removeAll();
      xScrollbar.removeAll();
      xScrollbar.add(magnifiableGraphicPane);
      yScrollbar.removeAll();
      yScrollbar.add(magnifiableGraphicPane);

      removeXLabel();
      removeYLabel();
      removeXScale();
      removeYScale();
      removeLegendLabel();

      invalidate();
   }


   // Implement the ScrollPositioner interface.
   /** Get the scroll position of the scrollable elements. */
   public PointUC getScrollPosition() {
      return new PointUC(
         graphicPane.xformXAWT2UC(xScrollbar.getPosition()),
         graphicPane.xformYAWT2UC(yScrollbar.getPosition())
      );
   }

   /** Set the scroll position of the scrollable elements. */
   public void setScrollPosition(PointUC p) {
      xScrollbar.setPosition(graphicPane.xformXUC2AWT(p.x));
      yScrollbar.setPosition(graphicPane.xformYUC2AWT(p.y));
      xScrollbar.repaintAdjusted();
      yScrollbar.repaintAdjusted();
   }


   /**
    * Override doLayout() to coordinate the layout of the GraphicPane
    * and the AxisScales.
    */
   public void doLayout() {
      graphicPane.updateScaleRange();
      super.doLayout();
   }


   /**
    * Override setLayout(LayoutManager) to ignore attemps 
    * to change the layout manager.
    */
   public void setLayout(LayoutManager l) {};
}   
